import time
import requests
from random import randint
import logging
from os import environ

radio_name = environ.get('radio_name', default='controlled_radio')
player_name = environ.get('player_name', default='controlled_player')
player_id = environ.get('playerid', default='6307651e660410d3678d1fa3')


class Player:
    def __init__(self, name):
        import docker

        self.name = name
        self.ct = None
        self.docker = docker

        logging.debug('Setting up docker sock')
        self.client = docker.DockerClient(base_url='unix:///var/run/docker.sock')

        self.check_exists()

    def start_new(self, stream, restart='no', labels=None):
        if labels is None:
            labels = {}
        if self.ct:
            logging.info('Not starting a new container under this class object because a container already exists')
            return

        if restart != 'no' and restart != 'always':
            logging.info('Not a valid restart policy, must be always or no')
            return

        logging.debug('Running new container')
        self.ct = self.client.containers.run('4grxfq/player',
                                             name=self.name,
                                             devices=['/dev/snd'],
                                             environment={'FF_URL': stream},
                                             restart_policy={'Name': restart},
                                             labels=labels,
                                             detach=True)

    def start(self):
        if not self.ct:
            logging.info('Cannot start container under this class because no container has been created yet')
            return

        logging.debug('Starting existing container object')
        self.ct.start()

    def stop(self):
        if not self.ct:
            logging.info('Not stopping container because no such object exists')
            return

        logging.debug('Stopping container')
        self.ct.stop()

    def remove(self):
        if not self.ct:
            logging.info('Not removing container because no such object exists')
            return

        logging.debug('Deleting container')
        self.ct.remove(force=True)
        self.ct = None

    def check_exists(self):
        try:
            logging.debug('Checking if container already exists under given name')
            self.ct = self.client.containers.get(self.name)
            return True
        except self.docker.errors.NotFound:
            logging.debug('Container does not exist continuing clean')
            self.ct = None
            return False

    def check_status(self):
        if not self.ct:
            logging.info('Cannot query container because no such object exists')
            return None

        self.ct.reload()
        if self.ct.status == 'running':
            logging.info('Container is running')
            return True
        else:
            logging.info('Container is not running')
            return False

    def get_labels(self):
        if not self.ct:
            logging.info('Cannot query container because no such object exists')
            return None

        self.ct.reload()
        return self.ct.labels


def request_config(p_id, stat):
    try:
        logging.debug('Requesting new config from controller')
        r = requests.post(f'http://192.168.99.20:5000/players/{p_id}/config', timeout=9, json=stat)

        if r.status_code != 200:
            logging.error('Config request not code 200, reporting')
            return None

        logging.debug(r.json())
        return r.json()

    except requests.exceptions.RequestException as exc:
        logging.error(f'Requesting config excepted: {exc}')


# def generate_status(play, radi):
def generate_status(radi):
    # give the player and radio objects to generate the status from
    # stat = {'radio': {}, 'player': {}}
    stat = {'radio': {}}

    # if play.check_status():  # if player container exists, it removes itself otherwise
    #     stat['player']['playing'] = True
    # else:
    #     stat['player']['playing'] = False
    #
    # if play.get_labels() is not None:
    #     stat['player']['labels']['stationId'] = play.get_labels().get('requestId')
    #     stat['player']['labels']['url'] = play.get_labels().get('url')
    #     stat['player']['labels']['cookie'] = play.get_labels().get('cookie')
    # else:
    #     stat['player']['labels'] = None

    if radi.check_status():  # check if radio is playing
        stat['radio']['playing'] = True
    else:
        stat['radio']['playing'] = False

    radi_labels = radi.get_labels()
    if radi_labels is not None:
        stat['radio']['stationId'] = radi_labels.get('stationId', '')
        stat['radio']['url'] = radi_labels.get('url', '')
        stat['radio']['cookie'] = radi_labels.get('cookie', '')

    logging.debug(stat)
    return stat


if __name__ == '__main__':
    radio = Player(radio_name)
    # player = Player(player_name)

    while True:
        # status = generate_status(player, radio)
        status = generate_status(radio)
        config = request_config(player_id, status)

        if config:
            if config.get('type') == 'radio':
                # player.remove()

                if config.get('stationId') == status['radio'].get('stationId'):
                    radio.start()
                    continue
                else:
                    radio.remove()
                    radio.start_new(config.get('url'), restart='always', labels={'url': config.get('url'),
                                                                                 'stationId': config.get('stationId'),
                                                                                 'cookie': config.get('cookie')})
                    continue

            # elif config.get('type') == 'request':
            #     radio.stop()
            #     player.start_new(config.get('url'), restart='no', labels={'url': config.get('url'),
            #                                                               'requestId': config.get('requestId'),
            #                                                               'cookie': config.get('cookie')})

            elif config.get('type') == 'silence':
                # player.remove()
                radio.stop()
                continue

        else:
            logging.debug('Nothing changed from previous request')

        logging.debug('Sleeping')
        time.sleep(randint(3, 6))