#!/usr/bin/env python3
import cmd, os, sys, time
from argon2 import PasswordHasher  # PIP: argon2-cffi
sys.tracebacklimit = 0  # disable tracebacks to hide potential secrets

ph = PasswordHasher()  # setup password hasher object with default parameters
hash = "$argon2id$v=19$m=65536,t=32,p=4$OvFXwX3buE9xYLKPGs2NOA$UEtxEwOI8VZ9WwWMGaySAhBdgP+zuLJIzCz4+Xt1YDw" example hash
peers = {'CORE1': {'endpoint': '1.1.1.1', 'status': True}, 'CORE2': {'endpoint': '1.1.1.2', 'status': False}}  # temporary testing dict


# Default root
class RootShell(cmd.Cmd):
    intro = f'Welcome {os.getlogin()} to my own shell for configuration.    Type help or ? to list nodes\n'
    prompt = '(root) '

    # Shell settings
    def default(self, arg):
        """ Return friendly string instead of bad syntax """
        print(f'Node does not exist, type help or ? to list commands.')

    def emptyline(self):
        """ Return empty """
        pass

    # Program functions
    def do_peer(self, arg):
        """
        Enter Wireguard peer configuration mode: PEER
        """
        Peer().cmdloop()  # execute Peer() subshell

    def do_benchmark(self, arg):
        """
        Benchmark Argon2id: BENCHMARk
        """
        benchmark()

    def do_exit(self, arg):
        """
        Exit and close session: EXIT
        """
        print('Goodbye')
        return True

# Example subshell
class Peer(cmd.Cmd):
    intro = 'Now entering Wireguard peer configuration mode'
    prompt = '(root/peer) '

    # Shell settings
    def default(self, arg):
        """ Return friendly string instead of bad syntax """
        print(f'Command does not exist, type help or ? to list commands.')

    def emptyline(self):
        """ Return empty """
        pass

    # Program functions
    def do_list(self, arg):
        """
        List peers that are setup with status: LIST
        """
        for x in peers:
            print(f'\n{x}')
            print(9*'=')
            print(f"Status: {peers[x]['status']}")
            print(f"Endpoint: {peers[x]['endpoint']}")
            print(9*'=')

    def do_add(self, arg):
        """
        Add peer configuration menu, make sure you specify a connection name which must me alphanumeric: ADD Conn1
        """
        if not arg:
            print('You need to specify a connection name')
            return
        elif not arg.isalnum():
            print('Name contains characters that are not alphanumeric')
            return

        print(f'Adding new connection under name: {arg}')

        try:
            endpoint = input('Endpoint(255.255.255.255): ')
        except KeyboardInterrupt:
            print('Aborting')
            return

        peers[arg] = {'endpoint': endpoint, 'status': False}
        print('Success')

    def do_delete(self, arg):
        """
        Delete peer connection: DELETE Conn1
        """
        if not arg:
            print('You need to specify a connection name')
            return
        elif not arg.isalnum():
            print('Name contains characters that are not alphanumeric')
            return
        elif arg not in peers:
            print('Connection does not exist')
            return

        peers.pop(arg)
        print('Success')

    def do_exit(self, arg):
        """
        Move up a level to root: EXIT
        """
        print("Movin' up")
        return True

# Login function, this is really insecure do not use, this was for argon2 testing
def login():
    n = 3
    while n:
        i = input('Please enter your unique key:\n')
        try:
            ph.verify(hash, i)
            valid = True
        except:
            n -= 1
            continue

        if ph.check_needs_rehash(hash):
            print('This key needs to be rehashed, please standby...')
            time.sleep(1)

        if valid:
            return True
    return False

# Quick argon2 benchmark for hasing speed
def benchmark():
    for x in range(2, 9, 2):  # for 2, 4, 6, 8 cores
        for n in range(2, 33, 2):  # for 2, 4, 6 ... 30, 32 time cost
            start = time.time()
            p = PasswordHasher(time_cost=n, parallelism=x, memory_cost=65536)
            p.hash('')
            end = time.time()
            print(f'Argon2id p={x}, t={n}, time={end-start}')


if __name__ == '__main__':
    #login
    if not login():
        print('Please return when you have the correct key')
        exit(0)

    try:
        RootShell().cmdloop()
    except KeyboardInterrupt:
        print('\nGoodbye')
        exit(0)
    except:
        print('An exception ocurred in shell level, please report this to the hostmaster')
        exit(1)